#include "Win32DxInterface.h"

#include "FileLogger.h"

#include <DirectXMath.h>
#include <DirectXPackedVector.h>

#include <dxgi.h>
#include <dxgi1_6.h>
#include <d3d12.h>
#include <d3d12sdklayers.h>

#include <iostream>
#include <vector>

#include <windows.h>
#include <wrl.h>

void Win32DxInterface::getHardwareAdapter(IDXGIAdapter1** ppAdapter)
{
	*ppAdapter = nullptr;
	for (UINT adapterIndex = 0; ; ++adapterIndex)
	{
		IDXGIAdapter1* pAdapter = nullptr;
		if (DXGI_ERROR_NOT_FOUND == mFactory->EnumAdapters1(adapterIndex, &pAdapter))
		{
			// No more adapters to enumerate.
			break;
		}

		// Check to see if the adapter supports Direct3D 12, but don't create the actual device yet.
		if (SUCCEEDED(D3D12CreateDevice(pAdapter, D3D_FEATURE_LEVEL_11_0, _uuidof(ID3D12Device), nullptr)))
		{
			*ppAdapter = pAdapter;
			return;
		}
		pAdapter->Release();
	}
}

ID3D12Device* Win32DxInterface::getDevice() const
{
	return mDevice.Get();
}

IDXGIFactory7* Win32DxInterface::getFactory() const
{
	return mFactory.Get();
}

void Win32DxInterface::initialize()
{
	if (!DirectX::XMVerifyCPUSupport())
	{
		MLOG_ERROR("Directx math not supported");
		return;
	}

#if defined(DEBUG) || defined(_DEBUG)
	// Enable the D3D12 debug layer.
	{
		ID3D12Debug* debugController;
		D3D12GetDebugInterface(IID_PPV_ARGS(&debugController));
		debugController->EnableDebugLayer();
	}
#endif

	HRESULT hr = CreateDXGIFactory(__uuidof(IDXGIFactory7), (void**)(&mFactory));
	if (hr == S_OK)
	{
		MLOG_INFO("FACTORY IS OK");
	}

	IDXGIAdapter1* pAdapter{ nullptr };
	getHardwareAdapter(&pAdapter);

	if (pAdapter)
	{
		MLOG_INFO("Found adapter");
		if (SUCCEEDED(D3D12CreateDevice(pAdapter, D3D_FEATURE_LEVEL_11_0, IID_PPV_ARGS(&mDevice))))
		{
			MLOG_INFO("Got device");
		}
	}
	else
	{
		MLOG_INFO("No adapter found");
		return;
	}
	return;
}